# matrix-appservice-kakaotalk - A Matrix-KakaoTalk puppeting bridge.
# Copyright (C) 2022 Tulir Asokan, Andrew Ferrazzutti
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
from typing import Optional

from attr import dataclass

from mautrix.types import JSON

from ..channel.channel_info import ChannelInfo, ChannelData
from .open_channel import OpenChannel
from . import OpenTokenComponent, OpenLink


@dataclass(kw_only=True)
class OpenChannelInfo(ChannelInfo, OpenChannel, OpenTokenComponent):
    directChannel: bool
    openLink: Optional[OpenLink] = None


@dataclass
class OpenChannelData(OpenChannel, ChannelData[OpenChannelInfo]):
    @classmethod
    def deserialize(cls, data: JSON) -> "OpenChannelData":
        data["info"] = OpenChannelInfo.deserialize(data["info"])
        return super().deserialize(data)


__all__ = [
    "OpenChannelInfo",
    "OpenChannelData",
]
